import { debounce } from 'src/common/runtime';
import { ExtractPropTypes } from 'vue';
import { ISharedRenderlessFunctionParams, ISharedRenderlessParamUtils } from './shared.type.js';

declare const tagGroupProps: {
    size: {
        type: StringConstructor;
        default: string;
        validator: (value: string) => boolean;
    };
    data: {
        type: ArrayConstructor;
        default: () => never[];
    };
    effect: {
        type: StringConstructor;
        default: string;
        validator: (value: string) => boolean;
    };
    tiny_mode: StringConstructor;
    tiny_mode_root: BooleanConstructor;
    tiny_template: (FunctionConstructor | ObjectConstructor)[];
    tiny_renderless: FunctionConstructor;
    tiny_theme: StringConstructor;
    tiny_mcp_config: ObjectConstructor;
    tiny_chart_theme: ObjectConstructor;
};

declare const handelItemClick: ({ emit }: Pick<ITagGroupRenderlessParamUtils, 'emit'>) => (item: ITagGroupDataItem, index: number, $event: MouseEvent) => void;
declare const getHiddenTags: ({ props, vm, state }: Pick<ITagGroupRenderlessParams, 'props' | 'vm' | 'state'>) => () => void;

interface ITagGroupDataItem {
    name: string;
    type?: string;
}
interface ITagGroupState {
    showMore: boolean;
    hiddenTags: ITagGroupDataItem[];
}
type ITagGroupRenderlessParams = ISharedRenderlessFunctionParams<never> & {
    api: ITagGroupApi;
    state: ITagGroupState;
    props: ITagGroupProps;
};
type ITagGroupProps = ExtractPropTypes<typeof tagGroupProps>;
interface ITagGroupApi {
    state: ITagGroupState;
    getHiddenTags: ReturnType<typeof getHiddenTags>;
    handelItemClick: ReturnType<typeof handelItemClick>;
    debouncedGetHiddenTags?: ReturnType<typeof debounce>;
}
type ITagGroupRenderlessParamUtils = ISharedRenderlessParamUtils<never>;

export { ITagGroupApi, ITagGroupDataItem, ITagGroupProps, ITagGroupRenderlessParamUtils, ITagGroupRenderlessParams, ITagGroupState };
